@@ -7,8 +7,10 @@ from tempfile import TemporaryFile, NamedTemporaryFile
 import wave
 import sys
 import struct
-from .logging_utils import log_conversion
+from .logging_utils import log_conversion, log_subprocess_output
+from .utils import mediainfo_json, fsdecode
 import base64
+from collections import namedtuple
 
 try:
     from StringIO import StringIO
@@ -69,6 +71,7 @@ class ClassPropertyDescriptor(object):
         self.fset = func
         return self
 
+
 def classproperty(func):
     if not isinstance(func, (classmethod, staticmethod)):
         func = classmethod(func)
@@ -81,6 +84,70 @@ AUDIO_FILE_EXT_ALIASES = {
     "wave": "wav",
 }
 
+WavSubChunk = namedtuple('WavSubChunk', ['id', 'position', 'size'])
+WavData = namedtuple('WavData', ['audio_format', 'channels', 'sample_rate',
+                                 'bits_per_sample', 'raw_data'])
+
+
+def extract_wav_headers(data):
+    # def search_subchunk(data, subchunk_id):
+    pos = 12  # The size of the RIFF chunk descriptor
+    subchunks = []
+    while pos + 8 <= len(data) and len(subchunks) < 10:
+        subchunk_id = data[pos:pos + 4]
+        subchunk_size = struct.unpack_from('<I', data[pos + 4:pos + 8])[0]
+        subchunks.append(WavSubChunk(subchunk_id, pos, subchunk_size))
+        if subchunk_id == b'data':
+            # 'data' is the last subchunk
+            break
+        pos += subchunk_size + 8
+
+    return subchunks
+
+
+def read_wav_audio(data, headers=None):
+    if not headers:
+        headers = extract_wav_headers(data)
+
+    fmt = [x for x in headers if x.id == b'fmt ']
+    if not fmt or fmt[0].size < 16:
+        raise CouldntDecodeError("Couldn't find fmt header in wav data")
+    fmt = fmt[0]
+    pos = fmt.position + 8
+    audio_format = struct.unpack_from('<H', data[pos:pos + 2])[0]
+    if audio_format != 1 and audio_format != 0xFFFE:
+        raise CouldntDecodeError("Unknown audio format 0x%X in wav data" %
+                                 audio_format)
+
+    channels = struct.unpack_from('<H', data[pos + 2:pos + 4])[0]
+    sample_rate = struct.unpack_from('<I', data[pos + 4:pos + 8])[0]
+    bits_per_sample = struct.unpack_from('<H', data[pos + 14:pos + 16])[0]
+
+    data_hdr = headers[-1]
+    if data_hdr.id != b'data':
+        raise CouldntDecodeError("Couldn't find data header in wav data")
+
+    pos = data_hdr.position + 8
+    return WavData(audio_format, channels, sample_rate, bits_per_sample,
+                   data[pos:pos + data_hdr.size])
+
+
+def fix_wav_headers(data):
+    headers = extract_wav_headers(data)
+    if not headers or headers[-1].id != b'data':
+        return
+
+    # TODO: Handle huge files in some other way
+    if len(data) > 2**32:
+        raise CouldntDecodeError("Unable to process >4GB files")
+
+    # Set the file size in the RIFF chunk descriptor
+    data[4:8] = struct.pack('<I', len(data) - 8)
+
+    # Set the data size in the data subchunk
+    pos = headers[-1].position
+    data[pos + 4:pos + 8] = struct.pack('<I', len(data) - pos - 8)
+
 
 class AudioSegment(object):
     """
@@ -116,6 +183,12 @@ class AudioSegment(object):
 
         audio_params = (self.sample_width, self.frame_rate, self.channels)
 
+        if isinstance(data, array.array):
+            try:
+                data = data.tobytes()
+            except:
+                data = data.tostring()
+
         # prevent partial specification of arguments
         if any(audio_params) and None in audio_params:
             raise MissingAudioParameter("Either all audio parameters or no parameter must be specified")
@@ -140,25 +213,24 @@ class AudioSegment(object):
                 data = data if isinstance(data, (basestring, bytes)) else data.read()
             except(OSError):
                 d = b''
-                reader = data.read(2**31-1)
+                reader = data.read(2 ** 31 - 1)
                 while reader:
                     d += reader
-                    reader = data.read(2**31-1)
+                    reader = data.read(2 ** 31 - 1)
                 data = d
 
-            raw = wave.open(StringIO(data), 'rb')
+            wav_data = read_wav_audio(data)
+            if not wav_data:
+                raise CouldntDecodeError("Couldn't read wav audio from data")
 
-            raw.rewind()
-            self.channels = raw.getnchannels()
-            self.sample_width = raw.getsampwidth()
-            self.frame_rate = raw.getframerate()
+            self.channels = wav_data.channels
+            self.sample_width = wav_data.bits_per_sample // 8
+            self.frame_rate = wav_data.sample_rate
             self.frame_width = self.channels * self.sample_width
-
-            raw.rewind()
-
-            # the "or b''" base case is a work-around for a python 3.4
-            # see https://github.com/jiaaro/pydub/pull/107
-            self._data = raw.readframes(float('inf')) or b''
+            self._data = wav_data.raw_data
+            if self.sample_width == 1:
+                # convert from unsigned integers in wav
+                self._data = audioop.bias(self._data, 1, -128)
 
         # Convert 24-bit audio to 32-bit audio.
         # (stdlib audioop and array modules do not support 24-bit data)
@@ -179,7 +251,6 @@ class AudioSegment(object):
                 old_bytes = struct.pack(pack_fmt, b0, b1, b2)
                 byte_buffer.write(old_bytes)
 
-
             self._data = byte_buffer.getvalue()
             self.sample_width = 4
             self.frame_width = self.channels * self.sample_width
@@ -193,12 +264,13 @@ class AudioSegment(object):
         """
         return self._data
 
-
-    def get_array_of_samples(self):
+    def get_array_of_samples(self, array_type_override=None):
         """
         returns the raw_data as an array of samples
         """
-        return array.array(self.array_type, self._data)
+        if array_type_override is None:
+            array_type_override = self.array_type
+        return array.array(array_type_override, self._data)
 
     @property
     def array_type(self):
@@ -216,6 +288,9 @@ class AudioSegment(object):
         except:
             return False
 
+    def __hash__(self):
+        return hash(AudioSegment) ^ hash((self.channels, self.frame_rate, self.sample_width, self._data))
+
     def __ne__(self, other):
         return not (self == other)
 
@@ -224,6 +299,12 @@ class AudioSegment(object):
 
     def __getitem__(self, millisecond):
         if isinstance(millisecond, slice):
+            if millisecond.step:
+                return (
+                    self[i:i + millisecond.step]
+                    for i in xrange(*millisecond.indices(len(self)))
+                )
+
             start = millisecond.start if millisecond.start is not None else 0
             end = millisecond.stop if millisecond.stop is not None \
                 else len(self)
@@ -351,22 +432,15 @@ class AudioSegment(object):
         return self.__class__(data=data, metadata=metadata)
 
     @classmethod
-    def _sync(cls, seg1, seg2):
-        s1_len, s2_len = len(seg1), len(seg2)
+    def _sync(cls, *segs):
+        channels = max(seg.channels for seg in segs)
+        frame_rate = max(seg.frame_rate for seg in segs)
+        sample_width = max(seg.sample_width for seg in segs)
 
-        channels = max(seg1.channels, seg2.channels)
-        seg1 = seg1.set_channels(channels)
-        seg2 = seg2.set_channels(channels)
-
-        frame_rate = max(seg1.frame_rate, seg2.frame_rate)
-        seg1 = seg1.set_frame_rate(frame_rate)
-        seg2 = seg2.set_frame_rate(frame_rate)
-
-        sample_width = max(seg1.sample_width, seg2.sample_width)
-        seg1 = seg1.set_sample_width(sample_width)
-        seg2 = seg2.set_sample_width(sample_width)
-
-        return seg1, seg2
+        return tuple(
+            seg.set_channels(channels).set_frame_rate(frame_rate).set_sample_width(sample_width)
+            for seg in segs
+        )
 
     def _parse_position(self, val):
         if val < 0:
@@ -398,9 +472,40 @@ class AudioSegment(object):
                                    "frame_width": 2})
 
     @classmethod
-    def from_file(cls, file, format=None, **kwargs):
+    def from_mono_audiosegments(cls, *mono_segments):
+        if not len(mono_segments):
+            raise ValueError("At least one AudioSegment instance is required")
+
+        segs = cls._sync(*mono_segments)
+
+        if segs[0].channels != 1:
+            raise ValueError(
+                "AudioSegment.from_mono_audiosegments requires all arguments are mono AudioSegment instances")
+
+        channels = len(segs)
+        sample_width = segs[0].sample_width
+        frame_rate = segs[0].frame_rate
+
+        frame_count = max(int(seg.frame_count()) for seg in segs)
+        data = array.array(
+            segs[0].array_type,
+            b'\0' * (frame_count * sample_width * channels)
+        )
+
+        for i, seg in enumerate(segs):
+            data[i::channels] = seg.get_array_of_samples()
+
+        return cls(
+            data,
+            channels=channels,
+            sample_width=sample_width,
+            frame_rate=frame_rate,
+        )
+
+    @classmethod
+    def from_file_using_temporary_files(cls, file, format=None, codec=None, parameters=None, **kwargs):
         orig_file = file
-        file = _fd_or_path_or_tempfile(file, 'rb', tempfile=False)
+        file, close_file = _fd_or_path_or_tempfile(file, 'rb', tempfile=False)
 
         if format:
             format = format.lower()
@@ -412,6 +517,122 @@ class AudioSegment(object):
                 return True
             if isinstance(orig_file, basestring):
                 return orig_file.lower().endswith(".{0}".format(f))
+            if isinstance(orig_file, bytes):
+                return orig_file.lower().endswith((".{0}".format(f)).encode('utf8'))
+            return False
+
+        if is_format("wav"):
+            try:
+                obj = cls._from_safe_wav(file)
+                if close_file:
+                    file.close()
+                return obj
+            except:
+                file.seek(0)
+        elif is_format("raw") or is_format("pcm"):
+            sample_width = kwargs['sample_width']
+            frame_rate = kwargs['frame_rate']
+            channels = kwargs['channels']
+            metadata = {
+                'sample_width': sample_width,
+                'frame_rate': frame_rate,
+                'channels': channels,
+                'frame_width': channels * sample_width
+            }
+            obj = cls(data=file.read(), metadata=metadata)
+            if close_file:
+                file.close()
+            return obj
+
+        input_file = NamedTemporaryFile(mode='wb', delete=False)
+        try:
+            input_file.write(file.read())
+        except(OSError):
+            input_file.flush()
+            input_file.close()
+            input_file = NamedTemporaryFile(mode='wb', delete=False, buffering=2 ** 31 - 1)
+            if close_file:
+                file.close()
+            close_file = True
+            file = open(orig_file, buffering=2 ** 13 - 1, mode='rb')
+            reader = file.read(2 ** 31 - 1)
+            while reader:
+                input_file.write(reader)
+                reader = file.read(2 ** 31 - 1)
+        input_file.flush()
+        if close_file:
+            file.close()
+
+        output = NamedTemporaryFile(mode="rb", delete=False)
+
+        conversion_command = [cls.converter,
+                              '-y',  # always overwrite existing files
+                              ]
+
+        # If format is not defined
+        # ffmpeg/avconv will detect it automatically
+        if format:
+            conversion_command += ["-f", format]
+
+        if codec:
+            # force audio decoder
+            conversion_command += ["-acodec", codec]
+
+        conversion_command += [
+            "-i", input_file.name,  # input_file options (filename last)
+            "-vn",  # Drop any video streams if there are any
+            "-f", "wav",  # output options (filename last)
+            output.name
+        ]
+
+        if parameters is not None:
+            # extend arguments with arbitrary set
+            conversion_command.extend(parameters)
+
+        log_conversion(conversion_command)
+
+        with open(os.devnull, 'rb') as devnull:
+            p = subprocess.Popen(conversion_command, stdin=devnull, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+        p_out, p_err = p.communicate()
+
+        log_subprocess_output(p_out)
+        log_subprocess_output(p_err)
+
+        try:
+            if p.returncode != 0:
+                raise CouldntDecodeError(
+                    "Decoding failed. ffmpeg returned error code: {0}\n\nOutput from ffmpeg/avlib:\n\n{1}".format(
+                        p.returncode, p_err))
+            obj = cls._from_safe_wav(output)
+        finally:
+            input_file.close()
+            output.close()
+            os.unlink(input_file.name)
+            os.unlink(output.name)
+
+        return obj
+
+    @classmethod
+    def from_file(cls, file, format=None, codec=None, parameters=None, **kwargs):
+        orig_file = file
+        try:
+            filename = fsdecode(file)
+        except TypeError:
+            filename = None
+        file, close_file = _fd_or_path_or_tempfile(file, 'rb', tempfile=False)
+
+        if format:
+            format = format.lower()
+            format = AUDIO_FILE_EXT_ALIASES.get(format, format)
+
+        def is_format(f):
+            f = f.lower()
+            if format == f:
+                return True
+
+            if filename:
+                return filename.lower().endswith(".{0}".format(f))
+
             return False
 
         if is_format("wav"):
@@ -431,22 +652,6 @@ class AudioSegment(object):
             }
             return cls(data=file.read(), metadata=metadata)
 
-        input_file = NamedTemporaryFile(mode='wb', delete=False)
-        try:
-            input_file.write(file.read())
-        except(OSError):
-            input_file.flush()
-            input_file.close()
-            input_file = NamedTemporaryFile(mode='wb', delete=False, buffering=2**31-1)
-            file = open(orig_file, buffering=2**13-1, mode='rb')
-            reader = file.read(2**31-1)
-            while reader:
-                input_file.write(reader)
-                reader = file.read(2**31-1)
-        input_file.flush()
-
-        output = NamedTemporaryFile(mode="rb", delete=False)
-
         conversion_command = [cls.converter,
                               '-y',  # always overwrite existing files
                               ]
@@ -456,69 +661,123 @@ class AudioSegment(object):
         if format:
             conversion_command += ["-f", format]
 
+        if codec:
+            # force audio decoder
+            conversion_command += ["-acodec", codec]
+
+        read_ahead_limit = kwargs.get('read_ahead_limit', -1)
+        if filename:
+            conversion_command += ["-i", filename]
+            stdin_parameter = None
+            stdin_data = None
+        else:
+            if cls.converter == 'ffmpeg':
+                conversion_command += ["-read_ahead_limit", str(read_ahead_limit),
+                                       "-i", "cache:pipe:0"]
+            else:
+                conversion_command += ["-i", "-"]
+            stdin_parameter = subprocess.PIPE
+            stdin_data = file.read()
+
+        if codec:
+            info = None
+        else:
+            info = mediainfo_json(orig_file, read_ahead_limit=read_ahead_limit)
+        if info:
+            audio_streams = [x for x in info['streams']
+                             if x['codec_type'] == 'audio']
+            # This is a workaround for some ffprobe versions that always say
+            # that mp3/mp4/aac/webm/ogg files contain fltp samples
+            audio_codec = audio_streams[0].get('codec_name')
+            if (audio_streams[0].get('sample_fmt') == 'fltp' and
+                    audio_codec in ['mp3', 'mp4', 'aac', 'webm', 'ogg']):
+                bits_per_sample = 16
+            else:
+                bits_per_sample = audio_streams[0]['bits_per_sample']
+            if bits_per_sample == 8:
+                acodec = 'pcm_u8'
+            else:
+                acodec = 'pcm_s%dle' % bits_per_sample
+
+            conversion_command += ["-acodec", acodec]
+
         conversion_command += [
-            "-i", input_file.name,  # input_file options (filename last)
             "-vn",  # Drop any video streams if there are any
             "-f", "wav",  # output options (filename last)
-            output.name
+            "-"
         ]
 
+        if parameters is not None:
+            # extend arguments with arbitrary set
+            conversion_command.extend(parameters)
+
         log_conversion(conversion_command)
 
-        p = subprocess.Popen(conversion_command, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
-        p_out, p_err = p.communicate()
+        p = subprocess.Popen(conversion_command, stdin=stdin_parameter,
+                             stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+        p_out, p_err = p.communicate(input=stdin_data)
 
-        if p.returncode != 0:
-            raise CouldntDecodeError("Decoding failed. ffmpeg returned error code: {0}\n\nOutput from ffmpeg/avlib:\n\n{1}".format(p.returncode, p_err))
+        if p.returncode != 0 or len(p_out) == 0:
+            if close_file:
+                file.close()
+            raise CouldntDecodeError(
+                "Decoding failed. ffmpeg returned error code: {0}\n\nOutput from ffmpeg/avlib:\n\n{1}".format(
+                    p.returncode, p_err))
 
-        obj = cls._from_safe_wav(output)
+        p_out = bytearray(p_out)
+        fix_wav_headers(p_out)
+        obj = cls._from_safe_wav(BytesIO(p_out))
 
-        input_file.close()
-        output.close()
-        os.unlink(input_file.name)
-        os.unlink(output.name)
+        if close_file:
+            file.close()
 
         return obj
 
     @classmethod
-    def from_mp3(cls, file):
-        return cls.from_file(file, 'mp3')
+    def from_mp3(cls, file, parameters=None):
+        return cls.from_file(file, 'mp3', parameters=parameters)
 
     @classmethod
-    def from_flv(cls, file):
-        return cls.from_file(file, 'flv')
+    def from_flv(cls, file, parameters=None):
+        return cls.from_file(file, 'flv', parameters=parameters)
 
     @classmethod
-    def from_ogg(cls, file):
-        return cls.from_file(file, 'ogg')
+    def from_ogg(cls, file, parameters=None):
+        return cls.from_file(file, 'ogg', parameters=parameters)
 
     @classmethod
-    def from_wav(cls, file):
-        return cls.from_file(file, 'wav')
+    def from_wav(cls, file, parameters=None):
+        return cls.from_file(file, 'wav', parameters=parameters)
 
     @classmethod
     def from_raw(cls, file, **kwargs):
-        return cls.from_file(file, 'raw', sample_width=kwargs['sample_width'], frame_rate=kwargs['frame_rate'], channels=kwargs['channels'])
+        return cls.from_file(file, 'raw', sample_width=kwargs['sample_width'], frame_rate=kwargs['frame_rate'],
+                             channels=kwargs['channels'])
 
     @classmethod
     def _from_safe_wav(cls, file):
-        file = _fd_or_path_or_tempfile(file, 'rb', tempfile=False)
+        file, close_file = _fd_or_path_or_tempfile(file, 'rb', tempfile=False)
         file.seek(0)
-        return cls(data=file)
+        obj = cls(data=file)
+        if close_file:
+            file.close()
+        return obj
 
-    def export(self, out_f=None, format='mp3', codec=None, bitrate=None, parameters=None, tags=None, id3v2_version='4', cover=None):
+    def export(self, out_f=None, format='mp3', codec=None, bitrate=None, parameters=None, tags=None, id3v2_version='4',
+               cover=None):
         """
         Export an AudioSegment to a file with given options
 
         out_f (string):
-            Path to destination audio file
+            Path to destination audio file. Also accepts os.PathLike objects on
+            python >= 3.6
 
         format (string)
             Format for destination audio file.
             ('mp3', 'wav', 'raw', 'ogg' or other ffmpeg/avconv supported files)
 
         codec (string)
-            Codec used to encoding for the destination.
+            Codec used to encode the destination file.
 
         bitrate (string)
             Bitrate used when encoding destination file. (64, 92, 128, 256, 312k...)
@@ -526,7 +785,7 @@ class AudioSegment(object):
             ffmpeg documentation for details (bitrate usually shown as -b, -ba or
             -a:b).
 
-        parameters (string)
+        parameters (list of strings)
             Aditional ffmpeg/avconv parameters
 
         tags (dict)
@@ -541,7 +800,13 @@ class AudioSegment(object):
         """
         id3v2_allowed_versions = ['3', '4']
 
-        out_f = _fd_or_path_or_tempfile(out_f, 'wb+')
+        if format == "raw" and (codec is not None or parameters is not None):
+            raise AttributeError(
+                    'Can not invoke ffmpeg when export format is "raw"; '
+                    'specify an ffmpeg raw format like format="s16le" instead '
+                    'or call export(format="raw") with no codec or parameters')
+
+        out_f, _ = _fd_or_path_or_tempfile(out_f, 'wb+')
         out_f.seek(0)
 
         if format == "raw":
@@ -549,12 +814,19 @@ class AudioSegment(object):
             out_f.seek(0)
             return out_f
 
-        # for wav output we can just write the data directly to out_f
-        if format == "wav":
+        # wav with no ffmpeg parameters can just be written directly to out_f
+        easy_wav = format == "wav" and codec is None and parameters is None
+
+        if easy_wav:
             data = out_f
         else:
             data = NamedTemporaryFile(mode="wb", delete=False)
 
+        pcm_for_wav = self._data
+        if self.sample_width == 1:
+            # convert to unsigned integers for wav
+            pcm_for_wav = audioop.bias(self._data, 1, 128)
+
         wave_data = wave.open(data, 'wb')
         wave_data.setnchannels(self.channels)
         wave_data.setsampwidth(self.sample_width)
@@ -562,11 +834,11 @@ class AudioSegment(object):
         # For some reason packing the wave header struct with
         # a float in python 2 doesn't throw an exception
         wave_data.setnframes(int(self.frame_count()))
-        wave_data.writeframesraw(self._data)
+        wave_data.writeframesraw(pcm_for_wav)
         wave_data.close()
 
-        # for wav files, we're done (wav data is written directly to out_f)
-        if format == 'wav':
+        # for easy wav files, we're done (wav data is written directly to out_f)
+        if easy_wav:
             return out_f
 
         output = NamedTemporaryFile(mode="w+b", delete=False)
@@ -583,9 +855,10 @@ class AudioSegment(object):
 
         if cover is not None:
             if cover.lower().endswith(('.png', '.jpg', '.jpeg', '.bmp', '.tif', '.tiff')) and format == "mp3":
-                conversion_command.extend(["-i" , cover, "-map", "0", "-map", "1", "-c:v", "mjpeg"])
+                conversion_command.extend(["-i", cover, "-map", "0", "-map", "1", "-c:v", "mjpeg"])
             else:
-                raise AttributeError("Currently cover images are only supported by MP3 files. The allowed image formats are: .tif, .jpg, .bmp, .jpeg and .png.")
+                raise AttributeError(
+                    "Currently cover images are only supported by MP3 files. The allowed image formats are: .tif, .jpg, .bmp, .jpeg and .png.")
 
         if codec is not None:
             # force audio encoder
@@ -614,10 +887,10 @@ class AudioSegment(object):
                         raise InvalidID3TagVersion(
                             "id3v2_version not allowed, allowed versions: %s" % id3v2_allowed_versions)
                     conversion_command.extend([
-                        "-id3v2_version",  id3v2_version
+                        "-id3v2_version", id3v2_version
                     ])
 
-        if sys.platform == 'darwin':
+        if sys.platform == 'darwin' and codec == 'mp3':
             conversion_command.extend(["-write_xing", "0"])
 
         conversion_command.extend([
@@ -627,11 +900,17 @@ class AudioSegment(object):
         log_conversion(conversion_command)
 
         # read stdin / write stdout
-        p = subprocess.Popen(conversion_command, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+        with open(os.devnull, 'rb') as devnull:
+            p = subprocess.Popen(conversion_command, stdin=devnull, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
         p_out, p_err = p.communicate()
 
+        log_subprocess_output(p_out)
+        log_subprocess_output(p_err)
+
         if p.returncode != 0:
-            raise CouldntEncodeError("Encoding failed. ffmpeg/avlib returned error code: {0}\n\nCommand:{1}\n\nOutput from ffmpeg/avlib:\n\n{2}".format(p.returncode, conversion_command, p_err))
+            raise CouldntEncodeError(
+                "Encoding failed. ffmpeg/avlib returned error code: {0}\n\nCommand:{1}\n\nOutput from ffmpeg/avlib:\n\n{2}".format(
+                    p.returncode, conversion_command, p_err))
 
         output.seek(0)
         out_f.write(output.read())
@@ -664,20 +943,12 @@ class AudioSegment(object):
         if sample_width == self.sample_width:
             return self
 
-        data = self._data
-
-        if self.sample_width == 1:
-            data = audioop.bias(data, 1, -128)
-
-        if data:
-            data = audioop.lin2lin(data, self.sample_width, sample_width)
-
-        if sample_width == 1:
-            data = audioop.bias(data, 1, 128)
-
         frame_width = self.channels * sample_width
-        return self._spawn(data, overrides={'sample_width': sample_width,
-                                            'frame_width': frame_width})
+
+        return self._spawn(
+            audioop.lin2lin(self._data, self.sample_width, sample_width),
+            overrides={'sample_width': sample_width, 'frame_width': frame_width}
+        )
 
     def set_frame_rate(self, frame_rate):
         if frame_rate == self.frame_rate:
@@ -700,11 +971,30 @@ class AudioSegment(object):
         if channels == 2 and self.channels == 1:
             fn = audioop.tostereo
             frame_width = self.frame_width * 2
+            fac = 1
+            converted = fn(self._data, self.sample_width, fac, fac)
         elif channels == 1 and self.channels == 2:
             fn = audioop.tomono
             frame_width = self.frame_width // 2
-
-        converted = fn(self._data, self.sample_width, 1, 1)
+            fac = 0.5
+            converted = fn(self._data, self.sample_width, fac, fac)
+        elif channels == 1:
+            channels_data = [seg.get_array_of_samples() for seg in self.split_to_mono()]
+            frame_count = int(self.frame_count())
+            converted = array.array(
+                channels_data[0].typecode,
+                b'\0' * (frame_count * self.sample_width)
+            )
+            for raw_channel_data in channels_data:
+                for i in range(frame_count):
+                    converted[i] += raw_channel_data[i] // self.channels
+            frame_width = self.frame_width // self.channels
+        elif self.channels == 1:
+            dup_channels = [self for iChannel in range(channels)]
+            return AudioSegment.from_mono_audiosegments(*dup_channels)
+        else:
+            raise ValueError(
+                "AudioSegment.set_channels only supports mono-to-multi channel and multi-to-mono channel conversion")
 
         return self._spawn(data=converted,
                            overrides={
@@ -715,22 +1005,26 @@ class AudioSegment(object):
         if self.channels == 1:
             return [self]
 
-        left_channel = audioop.tomono(self._data, self.sample_width, 1, 0)
-        right_channel = audioop.tomono(self._data, self.sample_width, 0, 1)
+        samples = self.get_array_of_samples()
 
-        return [self._spawn(data=left_channel,
-                            overrides={'channels': 1,
-                                       'frame_width': self.sample_width}),
-                self._spawn(data=right_channel,
-                            overrides={'channels': 1,
-                                       'frame_width': self.sample_width})]
+        mono_channels = []
+        for i in range(self.channels):
+            samples_for_current_channel = samples[i::self.channels]
+
+            try:
+                mono_data = samples_for_current_channel.tobytes()
+            except AttributeError:
+                mono_data = samples_for_current_channel.tostring()
+
+            mono_channels.append(
+                self._spawn(mono_data, overrides={"channels": 1, "frame_width": self.sample_width})
+            )
+
+        return mono_channels
 
     @property
     def rms(self):
-        if self.sample_width == 1:
-            return self.set_sample_width(2).rms
-        else:
-            return audioop.rms(self._data, self.sample_width)
+        return audioop.rms(self._data, self.sample_width)
 
     @property
     def dBFS(self):
@@ -818,7 +1112,7 @@ class AudioSegment(object):
         return self._spawn(data=audioop.mul(self._data, self.sample_width,
                                             db_to_float(float(volume_change))))
 
-    def overlay(self, seg, position=0, loop=False, times=None):
+    def overlay(self, seg, position=0, loop=False, times=None, gain_during_overlay=None):
         """
         Overlay the provided segment on to this segment starting at the
         specificed position and using the specfied looping beahvior.
@@ -838,6 +1132,10 @@ class AudioSegment(object):
             Loop seg the specified number of times or until it matches this
             segment's length. 1 means once, 2 means twice, ... 0 would make the
             call a no-op
+        gain_during_overlay (optional int):
+            Changes this segment's volume by the specified amount during the
+            duration of time that seg is overlaid on top of it. When negative,
+            this has the effect of 'ducking' the audio under the overlay.
         """
 
         if loop:
@@ -873,8 +1171,14 @@ class AudioSegment(object):
                 # is our last go-around
                 times = 1
 
-            output.write(audioop.add(seg1[pos:pos + seg2_len], seg2,
-                                     sample_width))
+            if gain_during_overlay:
+                seg1_overlaid = seg1[pos:pos + seg2_len]
+                seg1_adjusted_gain = audioop.mul(seg1_overlaid, self.sample_width,
+                                                 db_to_float(float(gain_during_overlay)))
+                output.write(audioop.add(seg1_adjusted_gain, seg2, sample_width))
+            else:
+                output.write(audioop.add(seg1[pos:pos + seg2_len], seg2,
+                                         sample_width))
             pos += seg2_len
 
             # dec times to break our while loop (eventually)
@@ -889,6 +1193,14 @@ class AudioSegment(object):
 
         if not crossfade:
             return seg1._spawn(seg1._data + seg2._data)
+        elif crossfade > len(self):
+            raise ValueError("Crossfade is longer than the original AudioSegment ({}ms > {}ms)".format(
+                crossfade, len(self)
+            ))
+        elif crossfade > len(seg):
+            raise ValueError("Crossfade is longer than the appended AudioSegment ({}ms > {}ms)".format(
+                crossfade, len(seg)
+            ))
 
         xf = seg1[-crossfade:].fade(to_gain=-120, start=0, end=float('inf'))
         xf *= seg2[:crossfade].fade(from_gain=-120, start=0, end=float('inf'))
@@ -900,7 +1212,9 @@ class AudioSegment(object):
         output.write(seg2[crossfade:]._data)
 
         output.seek(0)
-        return seg1._spawn(data=output)
+        obj = seg1._spawn(data=output)
+        output.close()
+        return obj
 
     def fade(self, to_gain=0, from_gain=0, start=None, end=None,
              duration=None):
@@ -965,7 +1279,7 @@ class AudioSegment(object):
 
         # fades longer than 100ms can use coarse fading (one gain step per ms),
         # shorter fades will have audible clicks so they use precise fading
-        #(one gain step per sample)
+        # (one gain step per sample)
         if duration > 100:
             scale_step = gain_delta / duration
 
@@ -1012,14 +1326,15 @@ class AudioSegment(object):
         )
 
     def _repr_html_(self):
-            src = """
+        src = """
                     <audio controls>
                         <source src="data:audio/mpeg;base64,{base64}" type="audio/mpeg"/>
                         Your browser does not support the audio element.
                     </audio>
                   """
-            fh = self.export()
-            data = base64.b64encode(fh.read()).decode('ascii')
-            return src.format(base64=data)
+        fh = self.export()
+        data = base64.b64encode(fh.read()).decode('ascii')
+        return src.format(base64=data)
+
 
 from . import effects
