@@ -2,16 +2,24 @@ from functools import partial
 import os
 import sys
 import unittest
-from tempfile import NamedTemporaryFile
+from tempfile import (
+    NamedTemporaryFile,
+    mkdtemp,
+    gettempdir
+)
+import tempfile
 import struct
 
 from pydub import AudioSegment
+from pydub.audio_segment import extract_wav_headers
 from pydub.utils import (
     db_to_float,
     ratio_to_db,
     make_chunks,
     mediainfo,
     get_encoder_name,
+    get_supported_decoders,
+    get_supported_encoders,
 )
 from pydub.exceptions import (
     InvalidTag,
@@ -50,6 +58,110 @@ class UtilityTests(unittest.TestCase):
         self.assertEqual(12, ratio_to_db(db_to_float(12, using_amplitude=False), using_amplitude=False))
 
 
+if sys.version_info >= (3, 6):
+    class PathLikeObjectTests(unittest.TestCase):
+
+        class MyPathLike:
+            def __init__(self, path):
+                self.path = path
+
+            def __fspath__(self):
+                return self.path
+
+        def setUp(self):
+            self.mp3_path_str = os.path.join(data_dir, 'test1.mp3')
+
+            from pathlib import Path
+            self.mp3_pathlib_path = Path(self.mp3_path_str)
+
+            self.mp3_path_like_str = self.MyPathLike(self.mp3_path_str)
+            self.mp3_path_like_bytes = self.MyPathLike(bytes(self.mp3_path_str, sys.getdefaultencoding()))
+
+        def test_audio_segment_from_pathlib_path(self):
+            seg1 = AudioSegment.from_file(self.mp3_path_str)
+            seg2 = AudioSegment.from_file(self.mp3_pathlib_path)
+
+            self.assertEqual(len(seg1), len(seg2))
+            self.assertEqual(seg1._data, seg2._data)
+            self.assertTrue(len(seg1) > 0)
+
+        def test_audio_segment_from_path_like_str(self):
+            seg1 = AudioSegment.from_file(self.mp3_path_str)
+            seg2 = AudioSegment.from_file(self.mp3_path_like_str)
+
+            self.assertEqual(len(seg1), len(seg2))
+            self.assertEqual(seg1._data, seg2._data)
+            self.assertTrue(len(seg1) > 0)
+
+        def test_audio_segment_from_path_like_bytes(self):
+            seg1 = AudioSegment.from_file(self.mp3_path_str)
+            seg2 = AudioSegment.from_file(self.mp3_path_like_bytes)
+
+            self.assertEqual(len(seg1), len(seg2))
+            self.assertEqual(seg1._data, seg2._data)
+            self.assertTrue(len(seg1) > 0)
+
+        def test_non_existant_pathlib_path(self):
+            from pathlib import Path
+            path = Path('this/path/should/not/exist/do/not/make/this/exist')
+            with self.assertRaises(FileNotFoundError):
+                _ = AudioSegment.from_file(path)
+
+            path = Path('')
+            # On Unicies this will raise a IsADirectoryError, on Windows this
+            # will result in a PermissionError. Both of these are subclasses of
+            # OSError. We aren't so much worried about the specific exception
+            # here, just that reading a file from an empty path is an error.
+            with self.assertRaises(OSError):
+                _ = AudioSegment.from_file(path)
+
+        def test_non_existant_path_like_str(self):
+            path = self.MyPathLike('this/path/should/not/exist/do/not/make/this/exist')
+            with self.assertRaises(FileNotFoundError):
+                _ = AudioSegment.from_file(path)
+
+            path = self.MyPathLike('')
+            with self.assertRaises(FileNotFoundError):
+                _ = AudioSegment.from_file(path)
+
+        def test_non_existant_path_like_bytes(self):
+            path = self.MyPathLike(bytes('this/path/should/not/exist/do/not/make/this/exist', sys.getdefaultencoding()))
+            with self.assertRaises(FileNotFoundError):
+                _ = AudioSegment.from_file(path)
+
+            path = self.MyPathLike(bytes('', sys.getdefaultencoding()))
+            with self.assertRaises(FileNotFoundError):
+                _ = AudioSegment.from_file(path)
+
+        def assertWithinRange(self, val, lower_bound, upper_bound):
+            self.assertTrue(lower_bound <= val <= upper_bound,
+                            "%s is not in the acceptable range: %s - %s" %
+                            (val, lower_bound, upper_bound))
+
+        def assertWithinTolerance(self, val, expected, tolerance=None,
+                                  percentage=None):
+            if percentage is not None:
+                tolerance = expected * percentage
+            lower_bound = expected - tolerance
+            upper_bound = expected + tolerance
+            self.assertWithinRange(val, lower_bound, upper_bound)
+
+        def test_export_pathlib_path(self):
+            seg1 = AudioSegment.from_file(self.mp3_path_str)
+            from pathlib import Path
+            path = Path(tempfile.gettempdir()) / 'pydub-test-export-8ajds.mp3'
+            try:
+                seg1.export(path, format='mp3')
+                seg2 = AudioSegment.from_file(path, format='mp3')
+
+                self.assertTrue(len(seg1) > 0)
+                self.assertWithinTolerance(len(seg1),
+                                           len(seg2),
+                                           percentage=0.01)
+            finally:
+                os.unlink(path)
+
+
 class FileAccessTests(unittest.TestCase):
 
     def setUp(self):
@@ -74,6 +186,7 @@ class AudioSegmentTests(unittest.TestCase):
     def setUp(self):
         global test1, test2, test3, testparty, testdcoffset
         if not test1:
+            a = os.path.join(data_dir, 'test1.mp3')
             test1 = AudioSegment.from_mp3(os.path.join(data_dir, 'test1.mp3'))
             test2 = AudioSegment.from_mp3(os.path.join(data_dir, 'test2.mp3'))
             test3 = AudioSegment.from_mp3(os.path.join(data_dir, 'test3.mp3'))
@@ -91,25 +204,27 @@ class AudioSegmentTests(unittest.TestCase):
         self.ogg_file_path = os.path.join(data_dir, 'bach.ogg')
         self.mp4_file_path = os.path.join(data_dir, 'creative_common.mp4')
         self.mp3_file_path = os.path.join(data_dir, 'party.mp3')
+        self.webm_file_path = os.path.join(data_dir, 'test5.webm')
 
         self.jpg_cover_path = os.path.join(data_dir, 'cover.jpg')
         self.png_cover_path = os.path.join(data_dir, 'cover.png')
 
     def assertWithinRange(self, val, lower_bound, upper_bound):
-        self.assertTrue(lower_bound < val < upper_bound,
+        self.assertTrue(lower_bound <= val <= upper_bound,
                         "%s is not in the acceptable range: %s - %s" %
                         (val, lower_bound, upper_bound))
 
     def assertWithinTolerance(self, val, expected, tolerance=None,
                               percentage=None):
         if percentage is not None:
-            tolerance = val * percentage
-        lower_bound = val - tolerance
-        upper_bound = val + tolerance
+            tolerance = expected * percentage
+        lower_bound = expected - tolerance
+        upper_bound = expected + tolerance
         self.assertWithinRange(val, lower_bound, upper_bound)
 
     def test_direct_instantiation_with_bytes(self):
-        seg = AudioSegment(b'RIFF\x28\x00\x00\x00WAVEfmt \x10\x00\x00\x00\x01\x00\x02\x00\x00}\x00\x00\x00\xf4\x01\x00\x04\x00\x10\x00data\x04\x00\x00\x00\x00\x00\x00\x00')
+        seg = AudioSegment(
+            b'RIFF\x28\x00\x00\x00WAVEfmt \x10\x00\x00\x00\x01\x00\x02\x00\x00}\x00\x00\x00\xf4\x01\x00\x04\x00\x10\x00data\x04\x00\x00\x00\x00\x00\x00\x00')
         self.assertEqual(seg.frame_count(), 1)
         self.assertEqual(seg.channels, 2)
         self.assertEqual(seg.sample_width, 2)
@@ -128,6 +243,57 @@ class AudioSegmentTests(unittest.TestCase):
         # the data length should have grown by exactly 4:3 (24 bits turn into 32 bits)
         self.assertEqual(len(seg24.raw_data) * 3, len24 * 4)
 
+    def test_8_bit_audio(self):
+        original_path = os.path.join(data_dir,'test1.wav')
+        original_segment = AudioSegment.from_file(original_path)
+        target_rms = original_segment.rms//2**8
+
+        path_with_8bits = os.path.join(data_dir,'test1-8bit.wav')
+
+        def check_8bit_segment(segment):
+            self.assertWithinTolerance(segment.rms,target_rms,tolerance=0)
+
+        # check reading directly
+        check_8bit_segment(AudioSegment.from_file(path_with_8bits))
+
+        # check using ffmpeg on it
+        with open(path_with_8bits,'rb') as file_8bit:
+            check_8bit_segment(AudioSegment.from_file(file_8bit))
+
+        # check conversion from higher-width sample
+        check_8bit_segment(AudioSegment.from_file(original_path).set_sample_width(1))
+
+        # check audio export
+        with NamedTemporaryFile('w+b', suffix='.wav') as tmp_file:
+            original_segment.set_sample_width(1).export(tmp_file,format='wav')
+            tmp_file.seek(0)
+            check_8bit_segment(AudioSegment.from_file(tmp_file))
+
+    def test_192khz_audio(self):
+        test_files = [('test-192khz-16bit.wav', 16),
+                      ('test-192khz-24bit.wav', 32),
+                      ('test-192khz-32bit.flac', 32),
+                      ('test-192khz-32bit.wav', 32),
+                      ('test-192khz-64bit.wav', 64)]
+        base_file, bit_depth = test_files[0]
+        path = os.path.join(data_dir, base_file)
+        base = AudioSegment.from_file(path)
+
+        headers = extract_wav_headers(open(path, 'rb').read())
+        data16_size = headers[-1].size
+        self.assertEqual(len(base.raw_data), data16_size)
+        self.assertEqual(base.frame_rate, 192000)
+        self.assertEqual(base.sample_width, bit_depth / 8)
+
+        for test_file, bit_depth in test_files[1:]:
+            path = os.path.join(data_dir, test_file)
+            seg = AudioSegment.from_file(path)
+            self.assertEqual(seg.sample_width, bit_depth / 8)
+            self.assertEqual(seg.frame_rate, 192000)
+            self.assertEqual(len(seg.raw_data), len(base.raw_data) *
+                             seg.sample_width / base.sample_width)
+            self.assertEqual(seg.frame_rate, 192000)
+
     def test_concat(self):
         catted_audio = self.seg1 + self.seg2
 
@@ -141,6 +307,11 @@ class AudioSegmentTests(unittest.TestCase):
         self.assertEqual(len(merged1), len(self.seg1) + len(self.seg3) - 100)
         self.assertEqual(len(merged2), len(self.seg2) + len(self.seg3) - 100)
 
+    def test_too_long_crossfade(self):
+        s1 = self.seg1[:1000]
+        s2 = self.seg2[:500]
+        self.assertRaises(ValueError, lambda: s1.append(s2, crossfade=len(s1) + 10))
+
     def test_sum(self):
         def gen():
             yield self.seg1
@@ -245,6 +416,21 @@ class AudioSegmentTests(unittest.TestCase):
         self.assertEqual(len(seg_over), 4000)
         self.assertFalse(seg_mult._data == seg_over._data)
 
+    def test_overlay_with_gain_change(self):
+        # Use overlay silence with volume change
+        seg_one = self.seg1[:5000]
+        seg_silent = AudioSegment.silent(duration=2000)
+        seg_over = seg_one.overlay(seg_silent, gain_during_overlay=-7)
+
+        # Manually lower first segment
+        seg_one_lower = seg_one - 7
+        seg_manual = seg_one_lower[:2000] + seg_one[2000:]
+
+        self.assertEqual(len(seg_over), len(seg_manual))
+        self.assertAlmostEqual(seg_over.dBFS, seg_manual.dBFS)
+        self.assertEqual(len(seg_manual), 5000)
+        self.assertEqual(len(seg_over), 5000)
+
     def test_slicing(self):
         empty = self.seg1[:0]
         second_long_slice = self.seg1[:1000]
@@ -263,6 +449,17 @@ class AudioSegmentTests(unittest.TestCase):
         past_end = second_long_slice[:1500]
         self.assertTrue(second_long_slice._data == past_end._data)
 
+    def test_slicing_by_step(self):
+        audio = self.seg1[:10000]
+        chunks = audio[:0]
+
+        for chunk in audio[::1000]:
+            self.assertTrue(isinstance(chunk, AudioSegment))
+            self.assertEqual(len(chunk), 1000)
+            chunks += chunk
+
+        self.assertEqual(len(audio), len(chunks))
+
     def test_indexing(self):
         short = self.seg1[:100]
 
@@ -289,9 +486,9 @@ class AudioSegmentTests(unittest.TestCase):
 
         monomp3 = AudioSegment.from_mp3(mono.export())
         self.assertWithinTolerance(len(monomp3), len(self.seg2),
-                                   percentage=0.01)
+                                   tolerance=105)
 
-        merged = monomp3.append(stereo, crossfade=100)
+        merged = mono.append(stereo, crossfade=100)
         self.assertWithinTolerance(len(merged),
                                    len(self.seg1) + len(self.seg2) - 100,
                                    tolerance=1)
@@ -311,7 +508,6 @@ class AudioSegmentTests(unittest.TestCase):
         self.assertEqual(seg_lchannel.frame_count(), seg.frame_count())
         self.assertEqual(seg_rchannel.frame_count(), seg.frame_count())
 
-
     def test_apply_gain_stereo(self):
         seg = self.seg1
 
@@ -390,15 +586,47 @@ class AudioSegmentTests(unittest.TestCase):
                                    len(seg),
                                    percentage=0.01)
 
+    def test_export_as_wav_with_codec(self):
+        seg = self.seg1
+        exported_wav = seg.export(format='wav', codec='pcm_s32le')
+        seg_exported_wav = AudioSegment.from_wav(exported_wav)
+
+        self.assertWithinTolerance(len(seg_exported_wav),
+                                   len(seg),
+                                   percentage=0.01)
+        self.assertEqual(seg_exported_wav.sample_width, 4)
+
+    def test_export_as_wav_with_parameters(self):
+        seg = self.seg1
+        exported_wav = seg.export(format='wav', parameters=['-ar', '16000', '-ac', '1'])
+        seg_exported_wav = AudioSegment.from_wav(exported_wav)
+
+        self.assertWithinTolerance(len(seg_exported_wav),
+                                   len(seg),
+                                   percentage=0.01)
+        self.assertEqual(seg_exported_wav.frame_rate, 16000)
+        self.assertEqual(seg_exported_wav.channels, 1)
+
     def test_export_as_raw(self):
         seg = self.seg1
         exported_raw = seg.export(format='raw')
-        seg_exported_raw = AudioSegment.from_raw(exported_raw, sample_width=seg.sample_width, frame_rate=seg.frame_rate, channels = seg.channels)
+        seg_exported_raw = AudioSegment.from_raw(exported_raw, sample_width=seg.sample_width, frame_rate=seg.frame_rate,
+                                                 channels=seg.channels)
 
         self.assertWithinTolerance(len(seg_exported_raw),
                                    len(seg),
                                    percentage=0.01)
 
+    def test_export_as_raw_with_codec(self):
+        seg = self.seg1
+        with self.assertRaises(AttributeError):
+            seg.export(format='raw', codec='pcm_s32le')
+
+    def test_export_as_raw_with_parameters(self):
+        seg = self.seg1
+        with self.assertRaises(AttributeError):
+            seg.export(format='raw', parameters=['-ar', '16000', '-ac', '1'])
+
     def test_export_as_ogg(self):
         seg = self.seg1
         exported_ogg = seg.export(format='ogg')
@@ -488,10 +716,12 @@ class AudioSegmentTests(unittest.TestCase):
             if sys.platform == 'win32':
                 tmp_mp3_file.close()
 
-            seg.export(tmp_mp3_file.name)
+            fd = seg.export(tmp_mp3_file.name)
+            fd.close()
 
             for i in range(3):
-                AudioSegment.from_mp3(tmp_mp3_file.name).export(tmp_mp3_file.name, "mp3")
+                fd = AudioSegment.from_mp3(tmp_mp3_file.name).export(tmp_mp3_file.name, "mp3")
+                fd.close()
 
             tmp_seg = AudioSegment.from_mp3(tmp_mp3_file.name)
             self.assertFalse(len(tmp_seg) < len(seg))
@@ -499,6 +729,8 @@ class AudioSegmentTests(unittest.TestCase):
             if sys.platform == 'win32':
                 os.remove(tmp_mp3_file.name)
 
+    @unittest.skipUnless('aac' in get_supported_decoders(),
+                         "Unsupported codecs")
     def test_formats(self):
         seg_m4a = AudioSegment.from_file(
             os.path.join(data_dir, 'format_test.m4a'), "m4a")
@@ -517,6 +749,8 @@ class AudioSegmentTests(unittest.TestCase):
         wav = AudioSegment.from_wav(wav_file)
         self.assertEqual(wav.duration_seconds, self.seg1.duration_seconds)
 
+    @unittest.skipUnless('aac' in get_supported_decoders(),
+                         "Unsupported codecs")
     def test_autodetect_format(self):
         aac_path = os.path.join(data_dir, 'wrong_extension.aac')
         fn = partial(AudioSegment.from_file, aac_path, "aac")
@@ -537,21 +771,41 @@ class AudioSegmentTests(unittest.TestCase):
             AudioSegment.from_file(self.mp3_file_path).export(tmp_ogg_file,
                                                               format="ogg")
 
+    def test_export_webm_as_mp3(self):
+        with NamedTemporaryFile('w+b', suffix='.mp3') as tmp_mp3_file:
+            AudioSegment.from_file(
+                self.webm_file_path,
+                codec="opus"
+            ).export(tmp_mp3_file, format="mp3")
+
+    def test_export_mp3_as_webm(self):
+        with NamedTemporaryFile('w+b', suffix='.webm') as tmp_webm_file:
+            AudioSegment.from_file(self.mp3_file_path).export(tmp_webm_file,
+                                                              format="webm")
+
+    @unittest.skipUnless('aac' in get_supported_decoders(),
+                         "Unsupported codecs")
     def test_export_mp4_as_ogg(self):
         with NamedTemporaryFile('w+b', suffix='.ogg') as tmp_ogg_file:
             AudioSegment.from_file(self.mp4_file_path).export(tmp_ogg_file,
                                                               format="ogg")
 
+    @unittest.skipUnless('aac' in get_supported_decoders(),
+                         "Unsupported codecs")
     def test_export_mp4_as_mp3(self):
         with NamedTemporaryFile('w+b', suffix='.mp3') as tmp_mp3_file:
             AudioSegment.from_file(self.mp4_file_path).export(tmp_mp3_file,
                                                               format="mp3")
 
+    @unittest.skipUnless('aac' in get_supported_decoders(),
+                         "Unsupported codecs")
     def test_export_mp4_as_wav(self):
         with NamedTemporaryFile('w+b', suffix='.wav') as tmp_wav_file:
             AudioSegment.from_file(self.mp4_file_path).export(tmp_wav_file,
                                                               format="mp3")
 
+    @unittest.skipUnless('aac' in get_supported_decoders(),
+                         "Unsupported codecs")
     def test_export_mp4_as_mp3_with_tags(self):
         with NamedTemporaryFile('w+b', suffix='.mp3') as tmp_mp3_file:
             tags_dict = {
@@ -563,6 +817,8 @@ class AudioSegmentTests(unittest.TestCase):
                                                               format="mp3",
                                                               tags=tags_dict)
 
+    @unittest.skipUnless('aac' in get_supported_decoders(),
+                         "Unsupported codecs")
     def test_export_mp4_as_mp3_with_tags_raises_exception_when_tags_are_not_a_dictionary(self):
         with NamedTemporaryFile('w+b', suffix='.mp3') as tmp_mp3_file:
             json = '{"title": "The Title You Want", "album": "Name of the Album", "artist": "Artist\'s name"}'
@@ -571,6 +827,8 @@ class AudioSegmentTests(unittest.TestCase):
                 format="mp3", tags=json)
             self.assertRaises(InvalidTag, func)
 
+    @unittest.skipUnless('aac' in get_supported_decoders(),
+                         "Unsupported codecs")
     def test_export_mp4_as_mp3_with_tags_raises_exception_when_id3version_is_wrong(self):
         tags = {'artist': 'Artist', 'title': 'Title'}
         with NamedTemporaryFile('w+b', suffix='.mp3') as tmp_mp3_file:
@@ -583,6 +841,8 @@ class AudioSegmentTests(unittest.TestCase):
             )
             self.assertRaises(InvalidID3TagVersion, func)
 
+    @unittest.skipUnless('aac' in get_supported_decoders(),
+                         "Unsupported codecs")
     def test_export_mp3_with_tags(self):
         tags = {'artist': 'Mozart', 'title': 'The Magic Flute'}
 
@@ -643,6 +903,15 @@ class AudioSegmentTests(unittest.TestCase):
         self.assertEqual(seg.frame_width, self.seg1.frame_width)
         self.assertEqual(seg.frame_rate, self.seg1.frame_rate)
 
+    def test_from_mono_audiosegments(self):
+        monoseg1 = self.seg1.set_channels(1)
+        monoseg2 = monoseg1.reverse()
+        stereo_sound = AudioSegment.from_mono_audiosegments(monoseg1, monoseg2)
+
+        self.assertEqual(stereo_sound.channels, 2)
+        self.assertEqual(stereo_sound.dBFS, monoseg1.dBFS)
+        self.assertEqual(len(stereo_sound), len(monoseg1))
+
     def test_fade_raises_exception_when_duration_is_negative(self):
         seg = self.seg1
         func = partial(seg.fade,
@@ -670,14 +939,14 @@ class AudioSegmentTests(unittest.TestCase):
         speedup_seg = self.seg1.speedup(2.0)
 
         self.assertWithinTolerance(
-            len(self.seg1) / 2, len(speedup_seg), percentage=0.01)
+            len(self.seg1) / 2, len(speedup_seg), percentage=0.02)
 
     def test_dBFS(self):
         seg_8bit = self.seg1.set_sample_width(1)
-        self.assertWithinTolerance(seg_8bit.dBFS, -8.88, tolerance=0.01)
-        self.assertWithinTolerance(self.seg1.dBFS, -8.88, tolerance=0.01)
-        self.assertWithinTolerance(self.seg2.dBFS, -10.39, tolerance=0.01)
-        self.assertWithinTolerance(self.seg3.dBFS, -6.47, tolerance=0.01)
+        self.assertWithinTolerance(seg_8bit.dBFS, -18.06, tolerance=1.5)
+        self.assertWithinTolerance(self.seg1.dBFS, -17.76, tolerance=1.5)
+        self.assertWithinTolerance(self.seg2.dBFS, -20.78, tolerance=1.5)
+        self.assertWithinTolerance(self.seg3.dBFS, -12.94, tolerance=1.5)
 
     def test_compress(self):
         compressed = self.seg1.compress_dynamic_range()
@@ -691,6 +960,8 @@ class AudioSegmentTests(unittest.TestCase):
         # average volume should be reduced
         self.assertTrue(compressed.rms < self.seg1.rms)
 
+    @unittest.skipUnless('aac' in get_supported_decoders(),
+                         "Unsupported codecs")
     def test_exporting_to_ogg_uses_default_codec_when_codec_param_is_none(self):
         delete = sys.platform != 'win32'
 
@@ -712,12 +983,37 @@ class AudioSegmentTests(unittest.TestCase):
         self.assertEqual(0, len(self.seg1[0:0]))
 
     def test_invert(self):
-        s = Sine(100).to_audio_segment()
+        s_mono = Sine(100).to_audio_segment()
+        s = s_mono.set_channels(2)
+
+        try:
+            s_mono.invert_phase(channels=(1, 0))
+        except Exception:
+            pass
+        else:
+            raise Exception("AudioSegment.invert_phase() didn't catch a bad input (mono)")
+
         s_inv = s.invert_phase()
         self.assertFalse(s == s_inv)
         self.assertTrue(s.rms == s_inv.rms)
         self.assertTrue(s == s_inv.invert_phase())
 
+        s_inv_right = s.invert_phase(channels=(0, 1))
+        left, right = s_inv_right.split_to_mono()
+
+        self.assertFalse(s_mono == s_inv_right)
+        self.assertFalse(s_inv == s_inv_right)
+        self.assertTrue(left == s_mono)
+        self.assertFalse(right == s_mono)
+
+        s_inv_left = s.invert_phase(channels=(1, 0))
+        left, right = s_inv_left.split_to_mono()
+
+        self.assertFalse(s_mono == s_inv_left)
+        self.assertFalse(s_inv == s_inv_left)
+        self.assertFalse(left == s_mono)
+        self.assertTrue(right == s_mono)
+
     def test_max_dBFS(self):
         sine_0_dbfs = Sine(1000).to_audio_segment()
         sine_minus_3_dbfs = Sine(1000).to_audio_segment(volume=-3.0)
@@ -766,6 +1062,25 @@ class AudioSegmentTests(unittest.TestCase):
         seg1 = seg.remove_dc_offset(channel=1, offset=(-0.06))
         self.assertWithinTolerance(seg1.get_dc_offset(1), -0.1, tolerance=0.01)
 
+    def test_from_file_clean_fail(self):
+        delete = sys.platform != 'win32'
+        orig_tmpdir = gettempdir()
+        new_tmpdir = mkdtemp()
+        tempfile.tempdir = new_tmpdir
+
+        with NamedTemporaryFile('w+b', suffix='.wav', delete=delete) as tmp_wav_file:
+            tmp_wav_file.write("not really a wav".encode('utf-8'))
+            tmp_wav_file.flush()
+            self.assertRaises(CouldntDecodeError, AudioSegment.from_file, tmp_wav_file.name)
+            files = os.listdir(tempfile.tempdir)
+            self.assertEquals(files, [os.path.basename(tmp_wav_file.name)])
+
+        if sys.platform == 'win32':
+            os.remove(tmp_wav_file.name)
+
+        tempfile.tempdir = orig_tmpdir
+        os.rmdir(new_tmpdir)
+
 
 class SilenceTests(unittest.TestCase):
 
@@ -798,6 +1113,11 @@ class SilenceTests(unittest.TestCase):
         silent_ranges = detect_silence(self.seg1, min_silence_len=500, silence_thresh=-20)
         self.assertEqual(silent_ranges, [[0, 775], [3141, 4033], [5516, 6051]])
 
+    def test_detect_silence_seg1_with_seek_split(self):
+        silent_ranges = detect_silence(self.seg1, min_silence_len=500, silence_thresh=-20,
+                                       seek_step=10)
+        self.assertEqual(silent_ranges, [[0, 770], [3150, 4030], [5520, 6050]])
+
     def test_realistic_audio(self):
         silent_ranges = detect_silence(self.seg4, min_silence_len=1000, silence_thresh=self.seg4.dBFS)
 
@@ -843,6 +1163,7 @@ class NoConverterTests(unittest.TestCase):
     def setUp(self):
         self.wave_file = os.path.join(data_dir, 'test1.wav')
         self.wave24_file = os.path.join(data_dir, 'test1-24bit.wav')
+        self.wave_empty = os.path.join(data_dir, 'test1_empty.wav')
         self.mp3_file = os.path.join(data_dir, 'test1.mp3')
         self.raw_file = os.path.join(data_dir, 'test1.raw')
         AudioSegment.converter = "definitely-not-a-path-to-anything-asdjklqwop"
@@ -904,7 +1225,7 @@ class NoConverterTests(unittest.TestCase):
         self.assertRaises(OSError, func)
 
     def test_init_AudioSegment_data_buffer(self):
-        seg = AudioSegment(data = "\0" * 34, sample_width=2, frame_rate=4, channels=1)
+        seg = AudioSegment(data="\0" * 34, sample_width=2, frame_rate=4, channels=1)
 
         self.assertEqual(seg.duration_seconds, 4.25)
 
@@ -912,31 +1233,39 @@ class NoConverterTests(unittest.TestCase):
 
         self.assertEqual(seg.frame_rate, 4)
 
-
     def test_init_AudioSegment_data_buffer_with_missing_args_fails(self):
-
-        func = partial(AudioSegment, data = "\0" * 16, sample_width=2, frame_rate=2)
+        func = partial(AudioSegment, data="\0" * 16, sample_width=2, frame_rate=2)
         self.assertRaises(MissingAudioParameter, func)
 
-        func = partial(AudioSegment, data = "\0" * 16, sample_width=2, channels=1)
+        func = partial(AudioSegment, data="\0" * 16, sample_width=2, channels=1)
         self.assertRaises(MissingAudioParameter, func)
 
-        func = partial(AudioSegment, data = "\0" * 16, frame_rate=2, channels=1)
+        func = partial(AudioSegment, data="\0" * 16, frame_rate=2, channels=1)
         self.assertRaises(MissingAudioParameter, func)
 
-
     def test_init_AudioSegment_data_buffer_with_bad_values_fails(self):
-
-        func = partial(AudioSegment, data = "\0" * 14, sample_width=4, frame_rate=2, channels=1)
+        func = partial(AudioSegment, data="\0" * 14, sample_width=4, frame_rate=2, channels=1)
         self.assertRaises(ValueError, func)
 
-
     def test_exporting(self):
         seg = AudioSegment.from_wav(self.wave_file)
         exported = AudioSegment.from_wav(seg.export(format="wav"))
 
         self.assertEqual(len(exported), len(seg))
 
+    def test_opening_empty_wav_file(self):
+        seg = AudioSegment.from_wav(self.wave_empty)
+        self.assertTrue(len(seg) == 0)
+
+        seg = AudioSegment.from_file(self.wave_empty)
+        self.assertTrue(len(seg) == 0)
+
+        seg = AudioSegment.from_file(self.wave_empty, "wav")
+        self.assertTrue(len(seg) == 0)
+
+        seg = AudioSegment.from_file(self.wave_empty, format="wav")
+        self.assertTrue(len(seg) == 0)
+
 
 class FilterTests(unittest.TestCase):
 
@@ -975,8 +1304,6 @@ class FilterTests(unittest.TestCase):
         self.assertAlmostEqual(less_treble.dBFS, s.dBFS, places=0)
 
 
-
-
 if __name__ == "__main__":
     import sys
 
